<?php

namespace Jigoshop\Service;

use Jigoshop\Entity\EntityInterface;
use Jigoshop\Entity\Product;

/**
 * Interface for Jigoshop services.
 *
 * @package Jigoshop\Service
 * @author  Amadeusz Starzykiewicz
 */
interface ServiceInterface
{
	/**
	 * Finds item specified by ID.
	 *
	 * @param $id int The ID.
	 *
	 * @return EntityInterface
	 */
	public function find($id);

	/**
	 * Finds item for specified WordPress post.
	 *
	 * @param $post \WP_Post WordPress post.
	 *
	 * @return EntityInterface Item found.
	 */
	public function findForPost($post);

	/**
	 * Finds items specified using WordPress query.
	 *
	 * @param $query \WP_Query WordPress query.
	 *
	 * @return Product[] Collection of found items.
	 */
	public function findByQuery($query);

	/**
	 * Saves entity to database.
	 *
	 * @param $object EntityInterface Entity to save.
	 */
	public function save(EntityInterface $object);
}